@@ -59,6 +59,8 @@
   * Utility library.
   */
 #include <vstring.h>
+#include <sock_addr.h>
+#include <myaddrinfo.h>
 
  /*
   * Structure for fixed resource record data.
@@ -81,7 +83,7 @@ typedef struct DNS_RR {
     unsigned int ttl;			/* always */
     unsigned short pref;		/* T_MX only */
     struct DNS_RR *next;		/* linkage */
-    unsigned data_len;			/* actual data size */
+    size_t data_len;			/* actual data size */
     char    data[1];			/* actually a bunch of data */
 } DNS_RR;
 
@@ -99,8 +101,9 @@ extern unsigned dns_type(const char *);
  /*
   * dns_rr.c
   */
-extern DNS_RR *dns_rr_create(const char *, DNS_FIXED *, unsigned,
-			             const char *, unsigned);
+extern DNS_RR *dns_rr_create(const char *, ushort, ushort,
+			             unsigned, unsigned,
+			             const char *, size_t);
 extern void dns_rr_free(DNS_RR *);
 extern DNS_RR *dns_rr_copy(DNS_RR *);
 extern DNS_RR *dns_rr_append(DNS_RR *, DNS_RR *);
@@ -108,13 +111,54 @@ extern DNS_RR *dns_rr_sort(DNS_RR *, int (*) (DNS_RR *, DNS_RR *));
 extern DNS_RR *dns_rr_shuffle(DNS_RR *);
 extern DNS_RR *dns_rr_remove(DNS_RR *, DNS_RR *);
 
+ /*
+  * dns_rr_to_pa.c
+  */
+extern const char *dns_rr_to_pa(DNS_RR *, MAI_HOSTADDR_STR *);
+
+ /*
+  * dns_sa_to_rr.c
+  */
+extern DNS_RR *dns_sa_to_rr(const char *, unsigned, struct sockaddr *);
+
+ /*
+  * dns_rr_to_sa.c
+  */
+extern int dns_rr_to_sa(DNS_RR *, unsigned, struct sockaddr *, SOCKADDR_SIZE *);
+
+ /*
+  * dns_rr_eq_sa.c
+  */
+extern int dns_rr_eq_sa(DNS_RR *, struct sockaddr *);
+
+#ifdef HAS_IPV6
+#define DNS_RR_EQ_SA(rr, sa) \
+    ((SOCK_ADDR_IN_FAMILY(sa) == AF_INET && (rr)->type == T_A \
+     && SOCK_ADDR_IN_ADDR(sa).s_addr == IN_ADDR((rr)->data).s_addr) \
+    || (SOCK_ADDR_IN_FAMILY(sa) == AF_INET6 && (rr)->type == T_AAAA \
+	&& memcmp((char *) &(SOCK_ADDR_IN6_ADDR(sa)), \
+		  (rr)->data, (rr)->data_len) == 0))
+#else
+#define DNS_RR_EQ_SA(rr, sa) \
+    (SOCK_ADDR_IN_FAMILY(sa) == AF_INET && (rr)->type == T_A \
+     && SOCK_ADDR_IN_ADDR(sa).s_addr == IN_ADDR((rr)->data).s_addr)
+#endif
+
  /*
   * dns_lookup.c
   */
 extern int dns_lookup(const char *, unsigned, unsigned, DNS_RR **,
 		              VSTRING *, VSTRING *);
-extern int dns_lookup_types(const char *, unsigned, DNS_RR **,
-			            VSTRING *, VSTRING *,...);
+extern int dns_lookup_l(const char *, unsigned, DNS_RR **, VSTRING *,
+			        VSTRING *, int,...);
+extern int dns_lookup_v(const char *, unsigned, DNS_RR **, VSTRING *,
+			        VSTRING *, int, unsigned *);
+
+ /*
+  * Request flags.
+  */
+#define DNS_REQ_FLAG_ANY	(1<<0)
+#define DNS_REQ_FLAG_ALL	(1<<1)
 
  /*
   * Status codes. Failures must have negative codes so they will not collide
