@@ -6,10 +6,9 @@
 /* SYNOPSIS
 /*	#include <mail_queue.h>
 /*
-/*	VSTREAM	*mail_queue_enter(queue_name, mode, tp)
+/*	VSTREAM	*mail_queue_enter(queue_name, mode)
 /*	const char *queue_name;
 /*	mode_t	mode;
-/*	struct timeval *tp;
 /*
 /*	VSTREAM	*mail_queue_open(queue_name, queue_id, flags, mode)
 /*	const char *queue_name;
@@ -54,9 +53,7 @@
 /*	id is the file base name, see VSTREAM_PATH().  Queue ids are
 /*	relatively short strings and are recycled in the course of time.
 /*	The only guarantee given is that on a given machine, no two queue
-/*	entries will have the same queue ID at the same time. The tp
-/*	argument, if not a null pointer, receives the time stamp that
-/*	corresponds with the queue ID.
+/*	entries will have the same queue ID at the same time.
 /*
 /*	mail_queue_open() opens the named queue file. The \fIflags\fR
 /*	and \fImode\fR arguments are as with open(2). The result is a
@@ -144,7 +141,7 @@
 const char *mail_queue_dir(VSTRING *buf, const char *queue_name,
 			           const char *queue_id)
 {
-    const char *myname = "mail_queue_dir";
+    char   *myname = "mail_queue_dir";
     static VSTRING *private_buf = 0;
     static VSTRING *hash_buf = 0;
     static ARGV *hash_queue_names = 0;
@@ -218,7 +215,7 @@ const char *mail_queue_path(VSTRING *buf, const char *queue_name,
 
 int     mail_queue_mkdirs(const char *path)
 {
-    const char *myname = "mail_queue_mkdirs";
+    char   *myname = "mail_queue_mkdirs";
     char   *saved_path = mystrdup(path);
     int     ret;
 
@@ -307,10 +304,9 @@ int     mail_queue_id_ok(const char *queue_id)
 
 /* mail_queue_enter - make mail queue entry with locally-unique name */
 
-VSTREAM *mail_queue_enter(const char *queue_name, mode_t mode,
-			          struct timeval * tp)
+VSTREAM *mail_queue_enter(const char *queue_name, mode_t mode)
 {
-    const char *myname = "mail_queue_enter";
+    char   *myname = "mail_queue_enter";
     static VSTRING *id_buf;
     static int pid;
     static VSTRING *path_buf;
@@ -330,8 +326,7 @@ VSTREAM *mail_queue_enter(const char *queue_name, mode_t mode,
 	path_buf = vstring_alloc(10);
 	temp_path = vstring_alloc(100);
     }
-    if (tp == 0)
-	tp = &tv;
+    GETTIMEOFDAY(&tv);
 
     /*
      * Create a file with a temporary name that does not collide. The process
@@ -343,13 +338,15 @@ VSTREAM *mail_queue_enter(const char *queue_name, mode_t mode,
      * If someone is racing against us, try to win.
      */
     for (;;) {
-	GETTIMEOFDAY(tp);
 	vstring_sprintf(temp_path, "%s/%d.%d", queue_name,
-			(int) tp->tv_usec, pid);
+			(int) tv.tv_usec, pid);
 	if ((fd = open(STR(temp_path), O_RDWR | O_CREAT | O_EXCL, mode)) >= 0)
 	    break;
-	if (errno == EEXIST || errno == EISDIR)
+	if (errno == EEXIST || errno == EISDIR) {
+	    if ((int) ++tv.tv_usec < 0)
+		tv.tv_usec = 0;
 	    continue;
+	}
 	msg_warn("%s: create file %s: %m", myname, STR(temp_path));
 	sleep(10);
     }
@@ -367,6 +364,7 @@ VSTREAM *mail_queue_enter(const char *queue_name, mode_t mode,
      * If someone is racing against us, try to win.
      */
     file_id = get_file_id(fd);
+    GETTIMEOFDAY(&tv);
 
     /*
      * XXX Some systems seem to have clocks that correlate with process
@@ -376,13 +374,22 @@ VSTREAM *mail_queue_enter(const char *queue_name, mode_t mode,
      * prevents multiple messages from getting the same Message-ID value.
      */
     for (count = 0;; count++) {
-	GETTIMEOFDAY(tp);
-	vstring_sprintf(id_buf, "%05X%s", (int) tp->tv_usec, file_id);
+	vstring_sprintf(id_buf, "%05X%s", (int) tv.tv_usec, file_id);
 	mail_queue_path(path_buf, queue_name, STR(id_buf));
+#if 0
+	if (access(STR(path_buf), X_OK) == 0) {	/* collision. */
+	    if ((int) ++tv.tv_usec < 0)
+		tv.tv_usec = 0;
+	    continue;
+	}
+#endif
 	if (sane_rename(STR(temp_path), STR(path_buf)) == 0)	/* success */
 	    break;
-	if (errno == EPERM || errno == EISDIR)	/* collision. weird. */
+	if (errno == EPERM || errno == EISDIR) {/* collision. weird. */
+	    if ((int) ++tv.tv_usec < 0)
+		tv.tv_usec = 0;
 	    continue;
+	}
 	if (errno != ENOENT || mail_queue_mkdirs(STR(path_buf)) < 0) {
 	    msg_warn("%s: rename %s to %s: %m", myname,
 		     STR(temp_path), STR(path_buf));
