@@ -20,15 +20,24 @@
   * Patches change both the patchlevel and the release date. Snapshots have no
   * patchlevel; they change the release date only.
   */
-#define MAIL_RELEASE_DATE	"20050627"
+#define MAIL_RELEASE_DATE	"20050716"
 #define MAIL_VERSION_NUMBER	"2.3"
 
-#define VAR_MAIL_VERSION	"mail_version"
 #ifdef SNAPSHOT
-#define DEF_MAIL_VERSION	MAIL_VERSION_NUMBER "-" MAIL_RELEASE_DATE
+# define MAIL_VERSION_DATE	"-" MAIL_RELEASE_DATE
 #else
-#define DEF_MAIL_VERSION	MAIL_VERSION_NUMBER
+# define MAIL_VERSION_DATE
 #endif
+
+#ifdef NONPROD
+# define MAIL_VERSION_PROD	"-nonprod"
+#else
+# define MAIL_VERSION_PROD
+#endif
+
+#define VAR_MAIL_VERSION	"mail_version"
+#define DEF_MAIL_VERSION	MAIL_VERSION_NUMBER MAIL_VERSION_DATE MAIL_VERSION_PROD
+
 extern char *var_mail_version;
 
  /*
