@@ -167,7 +167,6 @@
 
 #include <msg.h>
 #include <msg_syslog.h>
-#include <msg_vstream.h>
 #include <chroot_uid.h>
 #include <listen.h>
 #include <events.h>
@@ -250,7 +249,7 @@ static void multi_server_timeout(int unused_event, char *unused_context)
 
 /*  multi_server_drain - stop accepting new clients */
 
-int     multi_server_drain(void)
+int multi_server_drain(void)
 {
     int     fd;
 
@@ -260,7 +259,6 @@ int     multi_server_drain(void)
 	return (-1);
 	/* Finish existing clients in the background, then terminate. */
     case 0:
-	(void) msg_cleanup((MSG_CLEANUP_FN) 0);
 	for (fd = MASTER_LISTEN_FD; fd < MASTER_LISTEN_FD + socket_count; fd++)
 	    event_disable_readwrite(fd);
 	var_use_limit = 1;
@@ -372,7 +370,7 @@ static void multi_server_accept_local(int unused_event, char *context)
 	msg_fatal("select unlock: %m");
     if (fd < 0) {
 	if (errno != EAGAIN)
-	    msg_error("accept connection: %m");
+	    msg_fatal("accept connection: %m");
 	if (time_left >= 0)
 	    event_request_timer(multi_server_timeout, (char *) 0, time_left);
 	return;
@@ -409,7 +407,7 @@ static void multi_server_accept_pass(int unused_event, char *context)
 	msg_fatal("select unlock: %m");
     if (fd < 0) {
 	if (errno != EAGAIN)
-	    msg_error("accept connection: %m");
+	    msg_fatal("accept connection: %m");
 	if (time_left >= 0)
 	    event_request_timer(multi_server_timeout, (char *) 0, time_left);
 	return;
@@ -446,7 +444,7 @@ static void multi_server_accept_inet(int unused_event, char *context)
 	msg_fatal("select unlock: %m");
     if (fd < 0) {
 	if (errno != EAGAIN)
-	    msg_error("accept connection: %m");
+	    msg_fatal("accept connection: %m");
 	if (time_left >= 0)
 	    event_request_timer(multi_server_timeout, (char *) 0, time_left);
 	return;
@@ -458,7 +456,7 @@ static void multi_server_accept_inet(int unused_event, char *context)
 
 NORETURN multi_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
 {
-    const char *myname = "multi_server_main";
+    char   *myname = "multi_server_main";
     VSTREAM *stream = 0;
     char   *root_dir = 0;
     char   *user_name = 0;
@@ -474,18 +472,15 @@ NORETURN multi_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
     MAIL_SERVER_LOOP_FN loop = 0;
     int     key;
     char   *transport = 0;
-
 #if 0
     char   *lock_path;
     VSTRING *why;
-
 #endif
     int     alone = 0;
     int     zerolimit = 0;
     WATCHDOG *watchdog;
     char   *oval;
     char   *generation;
-    int     msg_vstream_needed = 0;
 
     /*
      * Process environment options as early as we can.
@@ -537,7 +532,7 @@ NORETURN multi_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
      * stderr, because no-one is going to see them.
      */
     opterr = 0;
-    while ((c = GETOPT(argc, argv, "cdDi:lm:n:o:s:St:uvVz")) > 0) {
+    while ((c = GETOPT(argc, argv, "cdDi:lm:n:o:s:St:uvz")) > 0) {
 	switch (c) {
 	case 'c':
 	    root_dir = "setme";
@@ -581,10 +576,6 @@ NORETURN multi_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
 	case 'v':
 	    msg_verbose++;
 	    break;
-	case 'V':
-	    if (++msg_vstream_needed == 1)
-		msg_vstream_init(mail_task(var_procname), VSTREAM_ERR);
-	    break;
 	case 'z':
 	    zerolimit = 1;
 	    break;
@@ -642,12 +633,12 @@ NORETURN multi_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
 	    multi_server_in_flow_delay = 1;
 	    break;
 	case MAIL_SERVER_SOLITARY:
-	    if (stream == 0 && !alone)
+	    if (!alone)
 		msg_fatal("service %s requires a process limit of 1",
 			  service_name);
 	    break;
 	case MAIL_SERVER_UNLIMITED:
-	    if (stream == 0 && !zerolimit)
+	    if (!zerolimit)
 		msg_fatal("service %s requires a process limit of 0",
 			  service_name);
 	    break;
@@ -755,7 +746,9 @@ NORETURN multi_server_main(int argc, char **argv, MULTI_SERVER_FN service,...)
      * Optionally, restrict the damage that this process can do.
      */
     resolve_local_init();
+#ifdef SNAPSHOT
     tzset();
+#endif
     chroot_uid(root_dir, user_name);
 
     /*
