@@ -158,7 +158,6 @@
 
 #include <msg.h>
 #include <msg_syslog.h>
-#include <msg_vstream.h>
 #include <chroot_uid.h>
 #include <vstring.h>
 #include <vstream.h>
@@ -262,7 +261,7 @@ static void trigger_server_wakeup(int fd)
 
 static void trigger_server_accept_fifo(int unused_event, char *context)
 {
-    const char *myname = "trigger_server_accept_fifo";
+    char   *myname = "trigger_server_accept_fifo";
     int     listen_fd = CAST_CHAR_PTR_TO_INT(context);
 
     if (trigger_server_lock != 0
@@ -286,7 +285,7 @@ static void trigger_server_accept_fifo(int unused_event, char *context)
 
 static void trigger_server_accept_local(int unused_event, char *context)
 {
-    const char *myname = "trigger_server_accept_local";
+    char   *myname = "trigger_server_accept_local";
     int     listen_fd = CAST_CHAR_PTR_TO_INT(context);
     int     time_left = 0;
     int     fd;
@@ -313,7 +312,7 @@ static void trigger_server_accept_local(int unused_event, char *context)
 	msg_fatal("select unlock: %m");
     if (fd < 0) {
 	if (errno != EAGAIN)
-	    msg_error("accept connection: %m");
+	    msg_fatal("accept connection: %m");
 	if (time_left >= 0)
 	    event_request_timer(trigger_server_timeout, (char *) 0, time_left);
 	return;
@@ -332,7 +331,7 @@ static void trigger_server_accept_local(int unused_event, char *context)
 
 static void trigger_server_accept_pass(int unused_event, char *context)
 {
-    const char *myname = "trigger_server_accept_pass";
+    char   *myname = "trigger_server_accept_pass";
     int     listen_fd = CAST_CHAR_PTR_TO_INT(context);
     int     time_left = 0;
     int     fd;
@@ -359,7 +358,7 @@ static void trigger_server_accept_pass(int unused_event, char *context)
 	msg_fatal("select unlock: %m");
     if (fd < 0) {
 	if (errno != EAGAIN)
-	    msg_error("accept connection: %m");
+	    msg_fatal("accept connection: %m");
 	if (time_left >= 0)
 	    event_request_timer(trigger_server_timeout, (char *) 0, time_left);
 	return;
@@ -378,7 +377,7 @@ static void trigger_server_accept_pass(int unused_event, char *context)
 
 NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,...)
 {
-    const char *myname = "trigger_server_main";
+    char   *myname = "trigger_server_main";
     char   *root_dir = 0;
     char   *user_name = 0;
     int     debug_me = 0;
@@ -404,7 +403,6 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
     WATCHDOG *watchdog;
     char   *oval;
     char   *generation;
-    int     msg_vstream_needed = 0;
 
     /*
      * Process environment options as early as we can.
@@ -456,7 +454,7 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
      * stderr, because no-one is going to see them.
      */
     opterr = 0;
-    while ((c = GETOPT(argc, argv, "cdDi:lm:n:o:s:St:uvVz")) > 0) {
+    while ((c = GETOPT(argc, argv, "cdDi:lm:n:o:s:St:uvz")) > 0) {
 	switch (c) {
 	case 'c':
 	    root_dir = "setme";
@@ -500,10 +498,6 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
 	case 'v':
 	    msg_verbose++;
 	    break;
-	case 'V':
-	    if (++msg_vstream_needed == 1)
-		msg_vstream_init(mail_task(var_procname), VSTREAM_ERR);
-	    break;
 	case 'z':
 	    zerolimit = 1;
 	    break;
@@ -558,12 +552,12 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
 	    trigger_server_in_flow_delay = 1;
 	    break;
 	case MAIL_SERVER_SOLITARY:
-	    if (stream == 0 && !alone)
+	    if (!alone)
 		msg_fatal("service %s requires a process limit of 1",
 			  service_name);
 	    break;
 	case MAIL_SERVER_UNLIMITED:
-	    if (stream == 0 && !zerolimit)
+	    if (!zerolimit)
 		msg_fatal("service %s requires a process limit of 0",
 			  service_name);
 	    break;
@@ -678,7 +672,9 @@ NORETURN trigger_server_main(int argc, char **argv, TRIGGER_SERVER_FN service,..
      * Optionally, restrict the damage that this process can do.
      */
     resolve_local_init();
+#ifdef SNAPSHOT
     tzset();
+#endif
     chroot_uid(root_dir, user_name);
 
     /*
