@@ -219,11 +219,8 @@
 /*	How much time a Postfix daemon process may take to handle a
 /*	request before it is terminated by a built-in watchdog timer.
 /* .IP "\fBdefer_transports (empty)\fR"
-/*	The names of message delivery transports that should not deliver mail
-/*	unless someone issues "\fBsendmail -q\fR" or equivalent.
-/* .IP "\fBdelay_logging_resolution_limit (2)\fR"
-/*	The maximal number of digits after the decimal point when logging
-/*	sub-second delay values.
+/*	The names of message delivery transports that should not be delivered
+/*	to unless someone issues "\fBsendmail -q\fR" or equivalent.
 /* .IP "\fBhelpful_warnings (yes)\fR"
 /*	Log warnings about problematic configuration settings, and provide
 /*	helpful suggestions.
@@ -328,6 +325,7 @@ int     var_local_rcpt_lim;		/* XXX */
 int     var_local_con_lim;		/* XXX */
 int     var_proc_limit;
 bool    var_verp_bounce_off;
+bool    var_sender_routing;
 int     var_qmgr_clog_warn_time;
 
 static QMGR_SCAN *qmgr_incoming;
@@ -558,6 +556,7 @@ int     main(int argc, char **argv)
     static CONFIG_BOOL_TABLE bool_table[] = {
 	VAR_ALLOW_MIN_USER, DEF_ALLOW_MIN_USER, &var_allow_min_user,
 	VAR_VERP_BOUNCE_OFF, DEF_VERP_BOUNCE_OFF, &var_verp_bounce_off,
+	VAR_SENDER_ROUTING, DEF_SENDER_ROUTING, &var_sender_routing,
 	0,
     };
 
