@@ -139,11 +139,6 @@ static NAME_MASK smtp_sasl_sec_mask[] = {
     0,
 };
 
- /*
-  * Silly little macros.
-  */
-#define STR(x)	vstring_str(x)
-
  /*
   * Macros to handle API differences between SASLv1 and SASLv2. Specifics:
   * 
