@@ -17,69 +17,28 @@
 /*	Where information is unavailable, the name and/or address
 /*	are set to "unknown".
 /*
-/*	This module uses the local name service via getaddrinfo()
-/*	and getnameinfo(). It does not query the DNS directly.
-/*
 /*	smtpd_peer_init() updates the following fields:
 /* .IP name
-/*	The verified client hostname. This name is represented by
-/*	the string "unknown" when 1) the address->name lookup failed,
-/*	2) the name->address mapping fails, or 3) the name->address
-/*	does not produce the client IP address.
-/* .IP reverse_name
-/*	The unverified client hostname as found with address->name
-/*	lookup; it is not verified for consistency with the client
-/*	IP address result from name->address lookup.
-/* .IP forward_name
-/*	The unverified client hostname as found with address->name
-/*	lookup followed by name->address lookup; it is not verified
-/*	for consistency with the result from address->name lookup.
-/*	For example, when the address->name lookup produces as
-/*	hostname an alias, the name->address lookup will produce
-/*	as hostname the expansion of that alias, so that the two
-/*	lookups produce different names.
+/*	The client hostname. An unknown name is represented by the
+/*	string "unknown". This includes names that could not be
+/*	verified with forward DNS lookups.
 /* .IP addr
 /*	Printable representation of the client address.
 /* .IP namaddr
 /*	String of the form: "name[addr]".
-/* .IP rfc_addr
-/*      String of the form "ipv4addr" or "ipv6:ipv6addr" for use
-/*	in Received: message headers.
-/* .IP name_status
-/*	The name_status result field specifies how the name
+/* .IP peer_code
+/*	The peer_code result field specifies how the client name
 /*	information should be interpreted:
 /* .RS
 /* .IP 2
-/*	The address->name lookup and name->address lookup produced
-/*	the client IP address.
+/*	Both name lookup and name verification succeeded.
 /* .IP 4
-/*	The address->name lookup or name->address lookup failed
-/*	with a recoverable error.
+/*	The name lookup or name verification failed with a recoverable
+/*	error (no address->name mapping or no name->address mapping).
 /* .IP 5
-/*	The address->name lookup or name->address lookup failed
-/*	with an unrecoverable error, or the result did not match
-/*	the client IP address.
-/* .RE
-/* .IP reverse_name_status
-/*	The reverse_name_status result field specifies how the
-/*	reverse_name information should be interpreted:
-/* .RS .IP 2
-/*	The address->name lookup succeeded.
-/* .IP 4
-/*	The address->name lookup failed with a recoverable error.
-/* .IP 5
-/*	The address->name lookup failed with an unrecoverable error.
-/* .RE .IP forward_name_status
-/*	The forward_name_status result field specifies how the
-/*	forward_name information should be interpreted:
-/* .RS .IP 2
-/*	The address->name and name->address lookup succeeded.
-/* .IP 4
-/*	The address->name lookup or name->address failed with a
-/*	recoverable error.
-/* .IP 5
-/*	The address->name lookup or name->address failed with an
-/*	unrecoverable error.
+/*	The name lookup or verification failed with an unrecoverable
+/*	error (no address->name mapping, bad hostname syntax, no
+/*	name->address mapping, client address not listed for hostname).
 /* .RE
 /* .PP
 /*	smtpd_peer_reset() releases memory allocated by smtpd_peer_init().
@@ -118,7 +77,6 @@
 
 #include <mail_proto.h>
 #include <valid_mailhost_addr.h>
-#include <mail_params.h>
 
 /* Application-specific. */
 
@@ -128,88 +86,45 @@
 
 void    smtpd_peer_init(SMTPD_STATE *state)
 {
-    const char *myname = "smtpd_peer_init";
-    SOCKADDR_SIZE sa_length;
+    char   *myname = "smtpd_peer_init";
+    SOCKADDR_SIZE sa_len;
     struct sockaddr *sa;
     INET_PROTO_INFO *proto_info = inet_proto_info();
 
     sa = (struct sockaddr *) & (state->sockaddr);
-    sa_length = sizeof(state->sockaddr);
+    sa_len = sizeof(state->sockaddr);
 
     /*
      * Look up the peer address information.
-     * 
-     * XXX If we make local endpoint (getsockname) information available to
-     * Milter applications as {if_name} and {if_addr}, then we also must be
-     * able to provide this via the XCLIENT command for Milter testing.
-     * 
-     * XXX If support were to be added for Milter applications in down-stream
-     * MTAs, then consistency demands that we propagate a lot of Sendmail
-     * macro information via the XFORWARD command. Otherwise we could end up
-     * with a very confusing situation.
      */
-    if (getpeername(vstream_fileno(state->client), sa, &sa_length) >= 0) {
+    if (getpeername(vstream_fileno(state->client), sa, &sa_len) >= 0) {
 	errno = 0;
     }
 
     /*
      * If peer went away, give up.
      */
-    if (errno != 0 && errno != ENOTSOCK) {
+    if (errno == ECONNRESET || errno == ECONNABORTED) {
 	state->name = mystrdup(CLIENT_NAME_UNKNOWN);
-	state->reverse_name = mystrdup(CLIENT_NAME_UNKNOWN);
 	state->addr = mystrdup(CLIENT_ADDR_UNKNOWN);
 	state->rfc_addr = mystrdup(CLIENT_ADDR_UNKNOWN);
-	state->addr_family = AF_UNSPEC;
-	state->name_status = SMTPD_PEER_CODE_PERM;
-	state->reverse_name_status = SMTPD_PEER_CODE_PERM;
+	state->peer_code = SMTPD_PEER_CODE_PERM;
     }
 
     /*
      * Convert the client address to printable address and hostname.
-     * 
-     * XXX If we're given an IPv6 (or IPv4) connection from, e.g., inetd, while
-     * Postfix IPv6 (or IPv4) support is turned off, don't (skip to the final
-     * else clause, pretend the origin is localhost[127.0.0.1], and become an
-     * open relay).
      */
     else if (errno == 0
-	     && (sa->sa_family == AF_INET
-#ifdef AF_INET6
-		 || sa->sa_family == AF_INET6
-#endif
-		 )) {
+	     && strchr((char *) proto_info->sa_family_list, sa->sa_family)) {
 	MAI_HOSTNAME_STR client_name;
 	MAI_HOSTADDR_STR client_addr;
 	int     aierr;
 	char   *colonp;
 
-	/*
-	 * Sanity check: we can't use sockets that we're not configured for.
-	 */
-	if (strchr((char *) proto_info->sa_family_list, sa->sa_family) == 0)
-	    msg_fatal("cannot handle socket type %s with \"%s = %s\"",
-#ifdef AF_INET6
-		      sa->sa_family == AF_INET6 ? "AF_INET6" :
-#endif
-		      sa->sa_family == AF_INET ? "AF_INET" :
-		      "other", VAR_INET_PROTOCOLS, var_inet_protocols);
-
-	/*
-	 * Sorry, but there are some things that we just cannot do while
-	 * connected to the network.
-	 */
-	if (geteuid() != var_owner_uid || getuid() != var_owner_uid) {
-	    msg_error("incorrect SMTP server privileges: uid=%lu euid=%lu",
-		      (unsigned long) getuid(), (unsigned long) geteuid());
-	    msg_fatal("the Postfix SMTP server must run with $%s privileges",
-		      VAR_MAIL_OWNER);
-	}
-
 	/*
 	 * Convert the client address to printable form.
 	 */
-	if ((aierr = sockaddr_to_hostaddr(sa, sa_length, &client_addr,
+	if ((aierr = sockaddr_to_hostaddr(sa, sa_len, &client_addr,
 					  (MAI_SERVPORT_STR *) 0, 0)) != 0)
 	    msg_fatal("%s: cannot convert client address to string: %s",
 		      myname, MAI_STRERROR(aierr));
@@ -233,15 +148,12 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 
 		state->addr = mystrdup(colonp + 1);
 		state->rfc_addr = mystrdup(colonp + 1);
-		state->addr_family = AF_INET;
 		aierr = hostaddr_to_sockaddr(state->addr, (char *) 0, 0, &res0);
 		if (aierr)
 		    msg_fatal("%s: cannot convert %s from string to binary: %s",
 			      myname, state->addr, MAI_STRERROR(aierr));
-		sa_length = res0->ai_addrlen;
-		if (sa_length > sizeof(state->sockaddr))
-		    sa_length = sizeof(state->sockaddr);
-		memcpy((char *) sa, res0->ai_addr, sa_length);
+		sa_len = res0->ai_addrlen;
+		memcpy((char *) sa, res0->ai_addr, sa_len);
 		freeaddrinfo(res0);		/* 200412 */
 	    }
 
@@ -257,7 +169,6 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 		state->addr = mystrdup(client_addr.buf);
 		state->rfc_addr =
 		    concatenate(IPV6_COL, client_addr.buf, (char *) 0);
-		state->addr_family = sa->sa_family;
 	    }
 	}
 
@@ -269,7 +180,6 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 	{
 	    state->addr = mystrdup(client_addr.buf);
 	    state->rfc_addr = mystrdup(client_addr.buf);
-	    state->addr_family = sa->sa_family;
 	}
 
 	/*
@@ -287,30 +197,20 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 #define REJECT_PEER_NAME(state, code) { \
 	myfree(state->name); \
 	state->name = mystrdup(CLIENT_NAME_UNKNOWN); \
-	state->name_status = code; \
+	state->peer_code = code; \
     }
 
-	if (var_smtpd_peername_lookup == 0) {
+	if ((aierr = sockaddr_to_hostname(sa, sa_len, &client_name,
+					  (MAI_SERVNAME_STR *) 0, 0)) != 0) {
 	    state->name = mystrdup(CLIENT_NAME_UNKNOWN);
-	    state->reverse_name = mystrdup(CLIENT_NAME_UNKNOWN);
-	    state->name_status = SMTPD_PEER_CODE_PERM;
-	    state->reverse_name_status = SMTPD_PEER_CODE_PERM;
-	} else if ((aierr = sockaddr_to_hostname(sa, sa_length, &client_name,
-					 (MAI_SERVNAME_STR *) 0, 0)) != 0) {
-	    state->name = mystrdup(CLIENT_NAME_UNKNOWN);
-	    state->reverse_name = mystrdup(CLIENT_NAME_UNKNOWN);
-	    state->name_status = (TEMP_AI_ERROR(aierr) ?
-			       SMTPD_PEER_CODE_TEMP : SMTPD_PEER_CODE_PERM);
-	    state->reverse_name_status = (TEMP_AI_ERROR(aierr) ?
-			       SMTPD_PEER_CODE_TEMP : SMTPD_PEER_CODE_PERM);
+	    state->peer_code = (TEMP_AI_ERROR(aierr) ?
+				SMTPD_PEER_CODE_TEMP : SMTPD_PEER_CODE_PERM);
 	} else {
 	    struct addrinfo *res0;
 	    struct addrinfo *res;
 
 	    state->name = mystrdup(client_name.buf);
-	    state->reverse_name = mystrdup(client_name.buf);
-	    state->name_status = SMTPD_PEER_CODE_OK;
-	    state->reverse_name_status = SMTPD_PEER_CODE_OK;
+	    state->peer_code = SMTPD_PEER_CODE_OK;
 
 	    /*
 	     * Reject the hostname if it does not list the peer address.
@@ -323,13 +223,13 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 		msg_warn("%s: hostname %s verification failed: %s",
 			 state->addr, state->name, MAI_STRERROR(aierr));
 		REJECT_PEER_NAME(state, (TEMP_AI_ERROR(aierr) ?
-			    SMTPD_PEER_CODE_TEMP : SMTPD_PEER_CODE_FORGED));
+			      SMTPD_PEER_CODE_TEMP : SMTPD_PEER_CODE_PERM));
 	    } else {
 		for (res = res0; /* void */ ; res = res->ai_next) {
 		    if (res == 0) {
 			msg_warn("%s: address not listed for hostname %s",
 				 state->addr, state->name);
-			REJECT_PEER_NAME(state, SMTPD_PEER_CODE_FORGED);
+			REJECT_PEER_NAME(state, SMTPD_PEER_CODE_PERM);
 			break;
 		    }
 		    if (strchr((char *) proto_info->sa_family_list, res->ai_family) == 0) {
@@ -351,12 +251,9 @@ void    smtpd_peer_init(SMTPD_STATE *state)
      */
     else {
 	state->name = mystrdup("localhost");
-	state->reverse_name = mystrdup("localhost");
 	state->addr = mystrdup("127.0.0.1");	/* XXX bogus. */
 	state->rfc_addr = mystrdup("127.0.0.1");/* XXX bogus. */
-	state->addr_family = AF_UNSPEC;
-	state->name_status = SMTPD_PEER_CODE_OK;
-	state->reverse_name_status = SMTPD_PEER_CODE_OK;
+	state->peer_code = SMTPD_PEER_CODE_OK;
     }
 
     /*
@@ -371,7 +268,6 @@ void    smtpd_peer_init(SMTPD_STATE *state)
 void    smtpd_peer_reset(SMTPD_STATE *state)
 {
     myfree(state->name);
-    myfree(state->reverse_name);
     myfree(state->addr);
     myfree(state->namaddr);
     myfree(state->rfc_addr);
