@@ -7,7 +7,7 @@
 /*	#include <vstream.h>
 /*
 /*	VSTREAM	*vstream_fopen(path, flags, mode)
-/*	const char *path;
+/*	char	*path;
 /*	int	flags;
 /*	mode_t	mode;
 /*
@@ -22,11 +22,11 @@
 /*	VSTREAM	*stream;
 /*
 /*	VSTREAM	*vstream_printf(format, ...)
-/*	const char *format;
+/*	char	*format;
 /*
 /*	VSTREAM	*vstream_fprintf(stream, format, ...)
 /*	VSTREAM	*stream;
-/*	const char *format;
+/*	char	*format;
 /*
 /*	int	VSTREAM_GETC(stream)
 /*	VSTREAM	*stream;
@@ -44,7 +44,7 @@
 /*	int	ch;
 /*
 /*	int	vstream_fputs(str, stream)
-/*	const char *str;
+/*	char	*str;
 /*	VSTREAM	*stream;
 /*
 /*	off_t	vstream_ftell(stream)
@@ -58,9 +58,6 @@
 /*	int	vstream_fflush(stream)
 /*	VSTREAM	*stream;
 /*
-/*	int	vstream_fpurge(stream)
-/*	VSTREAM	*stream;
-/*
 /*	ssize_t	vstream_fread(stream, buf, len)
 /*	VSTREAM	*stream;
 /*	char	*buf;
@@ -68,7 +65,7 @@
 /*
 /*	ssize_t	vstream_fwrite(stream, buf, len)
 /*	VSTREAM	*stream;
-/*	const char *buf;
+/*	char	*buf;
 /*	ssize_t	len;
 /*
 /*	void	vstream_control(stream, name, ...)
@@ -93,11 +90,11 @@
 /*	int	vstream_clearerr(stream)
 /*	VSTREAM	*stream;
 /*
-/*	const char *VSTREAM_PATH(stream)
+/*	char	*VSTREAM_PATH(stream)
 /*	VSTREAM	*stream;
 /*
 /*	char	*vstream_vfprintf(vp, format, ap)
-/*	const char *format;
+/*	char	*format;
 /*	va_list	*ap;
 /*
 /*	ssize_t	vstream_peek(stream)
@@ -112,9 +109,6 @@
 /*
 /*	time_t	vstream_ftime(stream)
 /*	VSTREAM	*stream;
-/*
-/*	struct timeval vstream_ftimeval(stream)
-/*	VSTREAM	*stream;
 /* DESCRIPTION
 /*	The \fIvstream\fR module implements light-weight buffered I/O
 /*	similar to the standard I/O routines.
@@ -216,12 +210,6 @@
 /*	vstream_fflush() returns 0 in case of success, VSTREAM_EOF in
 /*	case of problems. It is an error to flush a read-only stream.
 /*
-/*	vstream_fpurge() discards the contents of the stream buffer.
-/*	In the case of a double-buffered stream, it discards the
-/*	content of both the read and write buffers.
-/*	vstream_fpurge() returns 0 in case of success, VSTREAM_EOF in
-/*	case of problems.
-/*
 /*	vstream_fread() and vstream_fwrite() perform unformatted I/O
 /*	on the named stream. The result value is the number of bytes
 /*	transferred. A short count is returned in case of end-of-file
@@ -303,16 +291,12 @@
 /*	given to vstream_longjmp().
 /*
 /*	NB: non-local jumps such as vstream_longjmp() are not safe
-/*	for jumping out of any routine that manipulates VSTREAM data.
-/*	longjmp() like calls are best avoided in signal handlers.
+/*	for jumping out of any vstream routine.
 /*
 /*	vstream_ftime() returns the time of initialization, the last buffer
 /*	fill operation, or the last buffer flush operation for the specified
 /*	stream. This information is maintained only when stream timeouts are
 /*	enabled.
-/*
-/*	vstream_ftimeval() is like vstream_ftime() but returns more
-/*	detail.
 /* DIAGNOSTICS
 /*	Panics: interface violations. Fatal errors: out of memory.
 /* SEE ALSO
@@ -477,9 +461,8 @@ static void vstream_buf_init(VBUF *bp, int flags)
 
 static void vstream_buf_alloc(VBUF *bp, ssize_t len)
 {
-    VSTREAM *stream = VBUF_TO_APPL(bp, VSTREAM, buf);
     ssize_t used = bp->ptr - bp->data;
-    const char *myname = "vstream_buf_alloc";
+    char   *myname = "vstream_buf_alloc";
 
     if (len < bp->len)
 	msg_panic("%s: attempt to shrink buffer", myname);
@@ -493,15 +476,10 @@ static void vstream_buf_alloc(VBUF *bp, ssize_t len)
     bp->data = (unsigned char *)
 	(bp->data ? myrealloc((char *) bp->data, len) : mymalloc(len));
     bp->len = len;
-    if (bp->flags & VSTREAM_FLAG_READ) {
+    if (bp->flags & VSTREAM_FLAG_READ)
 	bp->ptr = bp->data + used;
-	if (bp->flags & VSTREAM_FLAG_DOUBLE)
-	    VSTREAM_SAVE_STATE(stream, read_buf, read_fd);
-    } else {
+    else
 	VSTREAM_BUF_AT_OFFSET(bp, used);
-	if (bp->flags & VSTREAM_FLAG_DOUBLE)
-	    VSTREAM_SAVE_STATE(stream, write_buf, write_fd);
-    }
 }
 
 /* vstream_buf_wipe - reset buffer to initial state */
@@ -518,7 +496,7 @@ static void vstream_buf_wipe(VBUF *bp)
 
 static int vstream_fflush_some(VSTREAM *stream, ssize_t to_flush)
 {
-    const char *myname = "vstream_fflush_some";
+    char   *myname = "vstream_fflush_some";
     VBUF   *bp = &stream->buf;
     ssize_t used;
     ssize_t left_over;
@@ -572,7 +550,7 @@ static int vstream_fflush_some(VSTREAM *stream, ssize_t to_flush)
 	    return (VSTREAM_EOF);
 	}
 	if (stream->timeout)
-	    GETTIMEOFDAY(&stream->iotime);
+	    stream->iotime = time((time_t *) 0);
 	if (msg_verbose > 2 && stream != VSTREAM_ERR && n != to_flush)
 	    msg_info("%s: %d flushed %ld/%ld", myname, stream->fd,
 		     (long) n, (long) to_flush);
@@ -631,7 +609,7 @@ static int vstream_fflush_delayed(VSTREAM *stream)
 static int vstream_buf_get_ready(VBUF *bp)
 {
     VSTREAM *stream = VBUF_TO_APPL(bp, VSTREAM, buf);
-    const char *myname = "vstream_buf_get_ready";
+    char   *myname = "vstream_buf_get_ready";
     ssize_t n;
 
     /*
@@ -669,8 +647,11 @@ static int vstream_buf_get_ready(VBUF *bp)
      * allocation gives the application a chance to override the default
      * buffering policy.
      */
-    if (bp->data == 0)
+    if (bp->data == 0) {
 	vstream_buf_alloc(bp, VSTREAM_BUFSIZE);
+	if (bp->flags & VSTREAM_FLAG_DOUBLE)
+	    VSTREAM_SAVE_STATE(stream, read_buf, read_fd);
+    }
 
     /*
      * If the stream is double-buffered and the write buffer is not empty,
@@ -705,7 +686,7 @@ static int vstream_buf_get_ready(VBUF *bp)
 	return (VSTREAM_EOF);
     default:
 	if (stream->timeout)
-	    GETTIMEOFDAY(&stream->iotime);
+	    stream->iotime = time((time_t *) 0);
 	if (msg_verbose > 2)
 	    msg_info("%s: fd %d got %ld", myname, stream->fd, (long) n);
 	bp->cnt = -n;
@@ -721,7 +702,7 @@ static int vstream_buf_get_ready(VBUF *bp)
 static int vstream_buf_put_ready(VBUF *bp)
 {
     VSTREAM *stream = VBUF_TO_APPL(bp, VSTREAM, buf);
-    const char *myname = "vstream_buf_put_ready";
+    char   *myname = "vstream_buf_put_ready";
 
     /*
      * Sanity checks. Detect a change of I/O direction or position. If so,
@@ -770,7 +751,7 @@ static int vstream_buf_space(VBUF *bp, ssize_t want)
     ssize_t used;
     ssize_t incr;
     ssize_t shortage;
-    const char *myname = "vstream_buf_space";
+    char   *myname = "vstream_buf_space";
 
     /*
      * Sanity checks. Reserving space implies writing. It is illegal to write
@@ -819,58 +800,11 @@ static int vstream_buf_space(VBUF *bp, ssize_t want)
     return (vstream_ferror(stream) ? VSTREAM_EOF : 0);	/* mmap() may fail */
 }
 
-/* vstream_fpurge - discard unread or unwritten content */
-
-int     vstream_fpurge(VSTREAM *stream)
-{
-    const char *myname = "vstream_fpurge";
-    VBUF   *bp = &stream->buf;
-
-    /*
-     * To discard all unread contents, position the read buffer at its end,
-     * so that we skip over any unread data, and so that the next read
-     * operation will refill the buffer.
-     * 
-     * To discard all unwritten content, position the write buffer at its
-     * beginning, so that the next write operation clobbers any unwritten
-     * data.
-     */
-    switch (bp->flags & (VSTREAM_FLAG_READ_DOUBLE | VSTREAM_FLAG_WRITE)) {
-    case VSTREAM_FLAG_READ_DOUBLE:
-	VSTREAM_BUF_AT_START(&stream->write_buf);
-	/* FALLTHROUGH */
-    case VSTREAM_FLAG_READ:
-	VSTREAM_BUF_AT_END(bp);
-	break;
-    case VSTREAM_FLAG_DOUBLE:
-	VSTREAM_BUF_AT_START(&stream->write_buf);
-	VSTREAM_BUF_AT_END(&stream->read_buf);
-	break;
-    case VSTREAM_FLAG_WRITE_DOUBLE:
-	VSTREAM_BUF_AT_END(&stream->read_buf);
-	/* FALLTHROUGH */
-    case VSTREAM_FLAG_WRITE:
-	VSTREAM_BUF_AT_START(bp);
-	break;
-    case VSTREAM_FLAG_READ_DOUBLE | VSTREAM_FLAG_WRITE:
-    case VSTREAM_FLAG_READ | VSTREAM_FLAG_WRITE:
-	msg_panic("%s: read/write stream", myname);
-    }
-
-    /*
-     * Invalidate the cached file seek position.
-     */
-    bp->flags &= ~VSTREAM_FLAG_SEEK;
-    stream->offset = 0;
-
-    return (0);
-}
-
 /* vstream_fseek - change I/O position */
 
 off_t   vstream_fseek(VSTREAM *stream, off_t offset, int whence)
 {
-    const char *myname = "vstream_fseek";
+    char   *myname = "vstream_fseek";
     VBUF   *bp = &stream->buf;
 
     /*
@@ -895,7 +829,6 @@ off_t   vstream_fseek(VSTREAM *stream, off_t offset, int whence)
 	    offset += bp->cnt;			/* subtract unread data */
 	else if (whence == SEEK_END)
 	    bp->flags &= ~VSTREAM_FLAG_SEEK;
-	/* FALLTHROUGH */
     case 0:
 	VSTREAM_BUF_AT_END(bp);
 	break;
@@ -1007,7 +940,7 @@ VSTREAM *vstream_fdopen(int fd, int flags)
     stream->timeout = 0;
     stream->context = 0;
     stream->jbuf = 0;
-    stream->iotime.tv_sec = stream->iotime.tv_usec = 0;
+    stream->iotime = 0;
     return (stream);
 }
 
@@ -1052,9 +985,8 @@ int     vstream_fclose(VSTREAM *stream)
      */
     if (stream->pid != 0)
 	msg_panic("vstream_fclose: stream has process");
-    if ((stream->buf.flags & VSTREAM_FLAG_WRITE_DOUBLE) != 0 && stream->fd >= 0)
+    if ((stream->buf.flags & VSTREAM_FLAG_WRITE_DOUBLE) != 0)
 	vstream_fflush(stream);
-    /* Do not remove: vstream_fdclose() depends on this error test. */
     err = vstream_ferror(stream);
     if (stream->buf.flags & VSTREAM_FLAG_DOUBLE) {
 	if (stream->read_fd >= 0)
@@ -1083,22 +1015,8 @@ int     vstream_fclose(VSTREAM *stream)
 
 int     vstream_fdclose(VSTREAM *stream)
 {
-
-    /*
-     * Flush unwritten output, just like vstream_fclose(). Errors are
-     * reported by vstream_fclose().
-     */
-    if ((stream->buf.flags & VSTREAM_FLAG_WRITE_DOUBLE) != 0)
-	(void) vstream_fflush(stream);
-
-    /*
-     * NOTE: Negative file descriptors are not part of the external
-     * interface. They are for internal use only, in order to support
-     * vstream_fdclose() without a lot of code duplication. Applications that
-     * rely on negative VSTREAM file descriptors will break without warning.
-     */
     if (stream->buf.flags & VSTREAM_FLAG_DOUBLE) {
-	stream->fd = stream->read_fd = stream->write_fd = -1;
+	stream->read_fd = stream->write_fd = -1;
     } else {
 	stream->fd = -1;
     }
@@ -1146,7 +1064,7 @@ int     vstream_fputs(const char *str, VSTREAM *stream)
 
 void    vstream_control(VSTREAM *stream, int name,...)
 {
-    const char *myname = "vstream_control";
+    char   *myname = "vstream_control";
     va_list ap;
 
     for (va_start(ap, name); name != VSTREAM_CTL_END; name = va_arg(ap, int)) {
@@ -1194,7 +1112,7 @@ void    vstream_control(VSTREAM *stream, int name,...)
 	    break;
 	case VSTREAM_CTL_TIMEOUT:
 	    if (stream->timeout == 0)
-		GETTIMEOFDAY(&stream->iotime);
+		stream->iotime = time((time_t *) 0);
 	    stream->timeout = va_arg(ap, int);
 	    break;
 	case VSTREAM_CTL_EXCEPT:
