@@ -14,7 +14,6 @@
  /*
   * System library.
   */
-#include <sys/time.h>
 #include <time.h>
 #include <fcntl.h>
 #include <stdarg.h>
@@ -49,7 +48,7 @@ typedef struct VSTREAM {
     VSTREAM_WAITPID_FN waitpid_fn;	/* vstream_popen/close() */
     int     timeout;			/* read/write timout */
     jmp_buf *jbuf;			/* exception handling */
-    struct timeval iotime;		/* time of last fill/flush */
+    time_t  iotime;			/* time of last fill/flush */
 } VSTREAM;
 
 extern VSTREAM vstream_fstd[];		/* pre-defined streams */
@@ -76,7 +75,6 @@ extern VSTREAM *vstream_fopen(const char *, int, mode_t);
 extern int vstream_fclose(VSTREAM *);
 extern off_t vstream_fseek(VSTREAM *, off_t, int);
 extern off_t vstream_ftell(VSTREAM *);
-extern int vstream_fpurge(VSTREAM *);
 extern int vstream_fflush(VSTREAM *);
 extern int vstream_fputs(const char *, VSTREAM *);
 extern VSTREAM *vstream_fdopen(int, int);
@@ -99,9 +97,8 @@ extern int vstream_fdclose(VSTREAM *);
 #define vstream_feof(vp)	vbuf_eof(&(vp)->buf)
 #define vstream_ftimeout(vp)	vbuf_timeout(&(vp)->buf)
 #define vstream_clearerr(vp)	vbuf_clearerr(&(vp)->buf)
-#define VSTREAM_PATH(vp)	((vp)->path ? (const char *) (vp)->path : "unknown_stream")
-#define vstream_ftime(vp)	((time_t) ((vp)->iotime.tv_sec))
-#define vstream_ftimeval(vp)	((vp)->iotime)
+#define VSTREAM_PATH(vp)	((vp)->path ? (vp)->path : "unknown_stream")
+#define vstream_ftime(vp)	((vp)->iotime)
 
 extern void vstream_control(VSTREAM *, int,...);
 
@@ -146,12 +143,6 @@ extern ssize_t vstream_peek(VSTREAM *);
 #define vstream_setjmp(stream)		setjmp((stream)->jbuf[0])
 #define vstream_longjmp(stream, val)	longjmp((stream)->jbuf[0], (val))
 
- /*
-  * Tweaks and workarounds.
-  */
-extern int vstream_tweak_sock(VSTREAM *);
-extern int vstream_tweak_tcp(VSTREAM *);
-
 /* LICENSE
 /* .ad
 /* .fi
